import logging
from flask_restful import Resource, reqparse

from orangeshare.devices import create_device, get_device

parser = reqparse.RequestParser()
parser.add_argument('name',
                    type=str,
                    required=True,
                    help='provide the name')


class NewDevice(Resource):
    """
    Adds a new device, sets the uuid, returns the content for the QR Code.
    """

    def post(self):
        args = parser.parse_args()
        name = args["name"]

        id = create_device(name)

        logging.info("Creating new device: \"{}\"".format(name))

        return get_device(id)



import argparse
import logging
import os
import sys
import threading
from typing import Optional

from orangeshare import Orangeshare


def get_args() -> argparse.Namespace:
    """
    Gets all commandline arguments and validates them.

    :return: The arguments
    """

    parser = argparse.ArgumentParser(description='A small python server that accepts requests from an apple shortcut to allow sharing all sorts of media from iOS with any desktop OS')
    parser.add_argument('--version', required=False, action='count', default=0, help="Print the Version")
    parser.add_argument('-p', '--api-port', required=False, type=int, help="Port number of the api server (default: 7615)", metavar="<port>", default=7615)
    parser.add_argument('-u', '--ui-port', required=False, type=int, help="Port number of the UI server (default: 7616)", metavar="<port>", default=7616)
    parser.add_argument('-o', '--open-ui', required=False, action='count', help="Open the server controls in the browser")
    if sys.platform == "win32":
        parser.add_argument('-t', '--tray-icon', required=False, action='count', help="Run with tray icon")
    parser.add_argument('-v', '--verbose', required=False, action='count', default=0, help="Enable verbose output")

    args = parser.parse_args()

    if args.version:
        from orangeshare import __version__
        print(__version__)
        exit(0)

    return args


orangeshare: Optional[Orangeshare] = None


def start_in_tray(api_port: int = 7615, ui_port: int = 7616, open_ui: bool = False):
    """
    Creates a tray icon and starts and stops Orange Share from its actions

    :param api_port: The port to run the API on
    :param ui_port: The port to run the UI on
    :param open_ui: If the UI should be opened each time
    """

    from pystray import Icon as icon, Menu as menu, MenuItem as item
    from PIL import Image

    image_active = Image.open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "logo/white.png"))
    image_inactive = Image.open(os.path.join(os.path.abspath(os.path.dirname(__file__)), "logo/gray.png"))

    def get_start_stop_text(icon):
        return "Stop" if orangeshare else "Start"

    def start_stop(icon, item):
        global orangeshare
        if orangeshare is None:
            icon.icon = image_active
            orangeshare = Orangeshare(api_port, ui_port)
            orangeshare.run(bool(open_ui))
        else:
            orangeshare.stop()
            orangeshare = None
            icon.icon = image_inactive

    def open_settings(icon, item):
        global orangeshare
        if orangeshare is None:
            icon.icon = image_active
            orangeshare = Orangeshare(api_port, ui_port)
            orangeshare.run()

        orangeshare.open_ui()

    def exit_app(icon, item):
        global orangeshare
        if orangeshare is not None:
            orangeshare.stop()

        icon.visible = False
        icon.stop()

    icon(
        'Orange Share',
        image_inactive,
        "Orange Share",
        menu=menu(
            item(
                get_start_stop_text,
                start_stop),
            item(
                "Settings",
                open_settings),
            item(
                "Exit",
                exit_app)
        )
    ).run()


def main():
    args = get_args()

    # set logging level
    logging.basicConfig(
        level=logging.DEBUG if args.verbose else logging.INFO,
        format='[%(asctime)s] %(levelname)-8s %(name)-12s %(message)s',
    )

    if sys.platform == "win32" and args.tray_icon:
        start_in_tray(args.api_port, args.ui_port, args.open_ui)
    else:
        orangeshare = Orangeshare(args.api_port, args.ui_port)
        orangeshare.run(bool(args.open_ui))


if __name__ == '__main__':
    main()



import logging
import socket
from typing import Optional

from appdirs import *
import configparser


class Config:
    _instance = None

    @staticmethod
    def get_config() -> 'Config':
        """
        Get the config (singleton)
        The config is loaded from file.
        If the file does not exist, default values will be set.

        :return:
        """

        if Config._instance is None:
            Config()
        return Config._instance

    def __init__(self):
        """
        Creates a new Config instance.
        Should not be called.
        Instead use get_config.
        """

        if Config._instance is not None:
            raise Exception("Config is a singleton")
        else:
            Config._instance = self

        self.directory = user_config_dir("orangeshare")
        self.file = os.path.join(self.directory, "config.ini")
        self._new_config = not os.path.isfile(self.file)

        self.config = configparser.ConfigParser()

        self._read()

        # will be set from main
        self.api_port: Optional[int] = None
        self.ui_port: Optional[int] = None

    def _read(self):
        """
        Loads the config from file.
        If the file does not exist, default values will be set.
        """

        logging.info("reading config file: {}".format(self.file))

        # get default values
        self.config.read(os.path.join(os.path.abspath(os.path.dirname(__file__)), "default.ini"))

        # get the actual values
        self.config.read(self.file)

        # save the config just in case new values were added
        self.save()

    def save(self):
        """
        Saves the config to.
        """

        os.makedirs(self.directory, exist_ok=True)

        with open(self.file, 'w') as configfile:
            self.config.write(configfile)

    @property
    def new_config(self) -> bool:
        """
        True if the config was newly created.
        This is the case when the application was run the first time.
        """

        return self._new_config



import base64
import json
import logging
import socket
import uuid
from typing import Dict

from orangeshare.config import Config


def check_credentials(username: str, password: str) -> bool:
    """
    Is used for the basic auth.
    Checks if a device with the an id identical to the passord exists.
    The username should always be "device"

    :param username: The username from the request
    :param password: The password from the request
    :return: True if a device with these credentials exists
    """

    if username != "device":
        logging.warning("Received request with wrong username \"{}\"".format(username))
        return False

    if Config.get_config().config.get("DEVICES", password, fallback=None) is None:
        logging.warning("Received request with unknown id")
        return False

    return True


def create_device(name: str) -> str:
    """
    Creates a new device with the given name
    The id will be set here.
    The device will be saved

    :param name: The name of the device
    :return: The id
    """

    id = str(uuid.uuid4())

    config = Config.get_config()
    config.config["DEVICES"][id] = name
    config.save()

    return id

def get_device(id):
    """
    The info for one device

    :param id: The device
    :return: The info
    """

    return {
        "name": Config.get_config().config.get("DEVICES", id),
        "id": id,
    }

def get_devices() -> Dict[str, str]:
    """
    Gets a list of devices from the config

    :return:
    """

    return {id: get_device(id) for id in Config.get_config().config["DEVICES"].keys()}


def delete_device(id: str):
    """
    Deletes the device with the given name from the config

    :param id: The ID of the device to delete
    """

    config = Config.get_config()
    config.config.remove_option("DEVICES", id)
    config.save()



import logging
import os
from typing import Optional

import wx

from orangeshare import Config
from orangeshare.notify import notify

file_dialog_open = False


def file_dialog(directory='', filename='', extension: Optional[str] = None) -> Optional[str]:
    """
    Opens a file dialog and returns the selected file.

    :param directory: The directory to open in
    :param filename: The default file name
    :param extension: The extension without .
    :return: The selected file or None
    """

    global file_dialog_open

    if file_dialog_open:
        # can't open second time
        # TODO: reasonable solution
        return None

    file_dialog_open = True

    wildcard = None
    if extension:
        wildcard = "*." + extension

    app = wx.App(None)
    dialog = wx.FileDialog(None, 'Open', wildcard=wildcard, style=wx.FD_SAVE, defaultDir=directory, defaultFile=filename)
    if dialog.ShowModal() == wx.ID_OK:
        path = dialog.GetPath()
    else:
        path = None
    dialog.Destroy()

    file_dialog_open = False
    return path


def handle_file(file, filename: str):
    """
    Copies the file to clipboard by saving it to a temporary directory and then copying it

    :param file: The file
    :param filename: The filename
    :return: response for the request
    """

    config = Config.get_config()
    if config.config.getboolean("SAVE", "notification", fallback=True):
        notify("Saving File \"{}\"".format(filename))

    extension = filename.split(".")[-1]
    if extension == filename:
        extension = ""

    if extension:
        f = file_dialog(directory=config.config.get("SAVE", "last_location", fallback=""), filename=filename, extension=extension)
    else:
        f = file_dialog(directory=config.config.get("SAVE", "last_location", fallback=""), filename=filename)
    if not f:
        logging.error("Error while saving file: No file destination was selected")
        return {"message": "No file destination was selected"}, 500

    if not f.endswith(extension):
        f += "." + extension

    config.config.set("SAVE", "last_location", os.path.dirname(os.path.abspath(f)))
    config.save()

    logging.info("saving File \"{}\"".format(f))

    file.save(f)

    return {'success': True}


def handle_text(*args):
    return {"message": "Saving is not available for text"}, 400



from os import path

from notifypy import Notify

def notify(content, title="Orange Share"):
    """
    Shows a notification.

    :param content: The content
    :param title: The titel (default: Orange Share)
    :return:
    """
    notification = Notify()
    notification.title = title
    notification.message = content
    notification.icon = path.join(path.abspath(path.dirname(__file__)), "logo/white.svg")
    notification.send(block=False)



"""
Orange-Share

A small python server that accepts requests from an apple shortcut to allow sharing all sorts of media from iOS with any desktop OS
"""

__version__ = "1.6.0"
__author__ = 'Yannis Vierkoetter'

import logging
import threading
from typing import Optional
from werkzeug.serving import make_server

from flask import Flask
from flask_basicauth import BasicAuth
from flask_restful import Api

from orangeshare import devices
from orangeshare.config import Config
from orangeshare.shortcuts import File, Text
import orangeshare.ui.api.devices
import orangeshare.ui.api.host
from orangeshare.shortcuts.get_data.GetData import GetData
from orangeshare.shortcuts.handlers.open_helper import open_url


class ServerThread(threading.Thread):
    def __init__(self, app, host, port):
        threading.Thread.__init__(self)

        self.port = port

        self.srv = make_server(host, port, app)
        self.ctx = app.app_context()
        self.ctx.push()

    def run(self):
        logging.info('starting server on port {}'.format(self.port))
        self.srv.serve_forever()

    def shutdown(self):
        logging.info('stopping server on port {}'.format(self.port))
        self.srv.shutdown()


class Orangeshare:
    """
    The Orange-Share Server
    """

    def __init__(self, api_port=7615, ui_port=7616):
        """
        Create a new Orange-Share instance.
        run still has to be called to start the server.

        :param api_port: The port to run the API on
        :param ui_port: The port to run the UI on
        """

        # The API server
        self.api_port = api_port
        Config.get_config().api_port = api_port

        self.api_app = Flask(__name__)

        basic_auth = BasicAuth(self.api_app)
        basic_auth.check_credentials = devices.check_credentials
        self.api_app.config['BASIC_AUTH_FORCE'] = True

        self.api_api = Api(self.api_app)

        # shortcuts
        self.api_api.add_resource(File.File, '/api/share/file')
        self.api_api.add_resource(Text.Text, '/api/share/text')
        self.api_api.add_resource(GetData, '/api/computer_data')

        # The UI Server
        self.ui_port = ui_port
        Config.get_config().ui_port = ui_port
        self.ui_app = Flask(__name__, template_folder="ui/templates", static_folder="ui/static")
        self.ui_api = Api(self.ui_app)

        # UI
        self.ui_app.add_url_rule("/", methods=["GET"], view_func=orangeshare.ui.index)
        self.ui_app.add_url_rule("/devices", methods=["GET"], view_func=orangeshare.ui.devices)
        self.ui_app.add_url_rule("/shortcuts", methods=["GET"], view_func=orangeshare.ui.shortcuts)
        self.ui_app.add_url_rule("/settings", methods=["GET"], view_func=orangeshare.ui.settings)
        self.ui_app.add_url_rule("/favicon.ico", methods=["GET"], view_func=orangeshare.ui.favicon)

        # API for the UI
        self.ui_api.add_resource(orangeshare.ui.api.devices.GetDevices, '/api/devices')
        self.ui_api.add_resource(orangeshare.ui.api.devices.NewDevice, '/api/devices/new')
        self.ui_api.add_resource(orangeshare.ui.api.devices.DeleteDevice, '/api/devices/delete')
        self.ui_api.add_resource(orangeshare.ui.api.host.Host, '/api/host')

        self.api_server: Optional[ServerThread] = None
        self.ui_server: Optional[ServerThread] = None

    def open_ui(self):
        """
        Opens the settings ui
        """

        open_url("http://localhost:{}".format(self.ui_port))

    def run(self, open_ui: bool = False):
        """
        Run the Orange-Share server

        :param open_ui: Open the controls in the browser once the server has started.
        """

        self.api_server = ServerThread(self.api_app, "0.0.0.0", self.api_port)
        self.api_server.start()

        self.ui_server = ServerThread(self.ui_app, "localhost", self.ui_port)
        self.ui_server.start()

        # open on first run or when specified
        if open_ui or Config.get_config().new_config:
            # threading.Timer(1, self.open_ui).start()
            self.open_ui()

    def stop(self):
        """
        Stop the server
        """
        self.api_server.shutdown()
        self.ui_server.shutdown()



import logging

from flask_restful import Resource, reqparse

import orangeshare.shortcuts.handlers.open
import orangeshare.shortcuts.handlers.clipboard

parser = reqparse.RequestParser()
parser.add_argument('mode',
                    type=str,
                    required=True,
                    help='provide a mode')
parser.add_argument('text',
                    type=str,
                    required=True,
                    help='provide the text')
parser.add_argument('filename',
                    type=str,
                    required=False,
                    help='provide the filename')
parser.add_argument('file_extension',
                    type=str,
                    required=True,
                    help='provide the file_extension')


class Text(Resource):
    """
    Receives a text from a shortcut and calls the respective function for open, save and clipboard
    """

    def post(self):
        args = parser.parse_args()
        text = args["text"]
        filename = args["filename"] if "filename" in args else None
        file_extension = args["file_extension"]

        preview = text.split("\n")[0][:50]
        if len(preview) < len(text):
            preview += "..."

        logging.info("Received Text \"{}\" for {}".format(preview, args["mode"]))

        if args["mode"] == "open":
            return orangeshare.shortcuts.handlers.open.handle_text(text, filename, file_extension)

        elif args["mode"] == "save":
            return orangeshare.shortcuts.handlers.save.handle_text(text, filename, file_extension)

        elif args["mode"] == "clipboard":
            return orangeshare.shortcuts.handlers.clipboard.handle_text(text, filename, file_extension)

        return {"message": "unknown mode \"{}\"".format(args["mode"])}, 400



from .New import NewDevice
from .Delete import DeleteDevice
from flask_restful import Resource, reqparse

from orangeshare.devices import get_devices


class GetDevices(Resource):
    """
    Adds a new device, sets the uuid, returns the content for the QR Code.
    """

    def get(self):
        return get_devices()



import pyperclip

from orangeshare import Config
from orangeshare.notify import notify


def handle_file(file: str, file_name: str):
    """
    Copies the file to clipboard by saving it to a temporary directory and then copying it

    :param file: The file
    :param file_name: The filename
    :return: response for the request
    """

    print(file, file_name)

    # config = Config.get_config()
    # if config.config.getboolean("CLIPBOARD", "notification", fallback=True):
    #     notify("Copied File to clipboard: \"{}\"".format(filename))

    # return {"success": True}
    return {"message": "Copying files to clipboard is not yet implemented"}


def handle_text(text: str, *args):
    """
    Copies the given text to the clipboard

    :param text: The text
    :return: response for the request
    """

    pyperclip.copy(text)

    config = Config.get_config()
    if config.config.getboolean("CLIPBOARD", "notification", fallback=True):
        if config.config.getboolean("CLIPBOARD", "notification_content", fallback=True):
            notify("Copied Text to clipboard:\n" + text)
        else:
            notify("Copied Text to clipboard")

    return {'success': True}



import sys

from setuptools import setup, find_packages
from os import path

this_directory = path.abspath(path.dirname(__file__))
with open(path.join(this_directory, 'README.md')) as f:
    long_description = f.read()

install_requires=[
    "flask",
    "flask_restful",
    "pyperclip",
    "notify-py",
    "validators",
    "wxpython",
    "appdirs",
    "Flask-BasicAuth",
    "netifaces",
    "pillow",
    "werkzeug"
]

if sys.platform == "win32":
    install_requires.append("pystray")

setup(
    name='orangeshare',
    version='1.6.0',
    description='A small python server that accepts requests from an apple shortcut to allow sharing all sorts of media from iOS with any desktop OS',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/Yannis4444/Orange-Share',
    author='Yannis Vierkoetter',
    author_email='yannis44@web.de',
    license='MIT',
    packages=find_packages(),
    include_package_data=True,
    install_requires=install_requires,
    classifiers=[
        'Intended Audience :: End Users/Desktop',
        'License :: OSI Approved :: MIT License',
        'Framework :: Flask',
        'Natural Language :: English',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3',
        'Topic :: Communications :: File Sharing'
    ],
    scripts=['orangeshare/orangeshare'],
)



import locale
import logging
import os
import threading
from typing import Optional

import pyperclip
import wx
from flask import make_response, send_file
from flask_restful import Resource

from orangeshare import Config


class DropTarget(wx.FileDropTarget):
    def __init__(self, obj, result_data):
        wx.FileDropTarget.__init__(self)
        self.obj = obj
        self.result_data = result_data

    def OnDropFiles(self, x, y, filenames):
        logging.info("{} dropped into get data dialog".format(filenames[0]))

        self.result_data["filename"] = filenames[0]

        self.obj.Parent.Close()

        return True


class GetDataFrame(wx.Frame):

    def __init__(self, parent, ID, title, result_data):
        """
        :param result_data: The dictionary to write the data to
        """
        wx.Frame.__init__(self, parent, ID, title, style=wx.DEFAULT_FRAME_STYLE ^ wx.RESIZE_BORDER)

        self.result_data = result_data
        self.dialog: Optional[wx.FileDialog] = None

        self.dark_mode = sum(self.GetBackgroundColour()[0:3]) < 128 * 3

        # destroy the window after 60 s as the Shortcut will time out at that time
        threading.Timer(60, self.try_destroying).start()

        ico = wx.Icon(os.path.join(os.path.dirname(__file__), "../../logo/white.ico"), wx.BITMAP_TYPE_ICO)
        self.SetIcon(ico)
        self.Show()

        # file icon
        file_panel = wx.Panel(self, -1, style=wx.NO_BORDER)
        file_panel.SetCursor(wx.Cursor(wx.CURSOR_HAND))
        bmp1 = wx.Image(os.path.join(os.path.dirname(__file__), "icons/file_dark_mode.png" if self.dark_mode else "icons/file_light_mode.png"), wx.BITMAP_TYPE_ANY).Scale(200, 200, wx.IMAGE_QUALITY_HIGH).ConvertToBitmap()
        file_bitmap = wx.StaticBitmap(file_panel, -1, bmp1, (0, 0))

        file_bitmap.Bind(wx.EVT_LEFT_UP, self.on_file_click)

        self.file_drop_target = DropTarget(file_panel, result_data)
        file_panel.SetDropTarget(self.file_drop_target)

        # clipboard icon
        clipboard_panel = wx.Panel(self, -1, style=wx.NO_BORDER)
        clipboard_panel.SetCursor(wx.Cursor(wx.CURSOR_HAND))
        bmp2 = wx.Image(os.path.join(os.path.dirname(__file__), "icons/clipboard_dark_mode.png" if self.dark_mode else "icons/clipboard_light_mode.png"), wx.BITMAP_TYPE_ANY).Scale(200, 200, wx.IMAGE_QUALITY_HIGH).ConvertToBitmap()
        file_bitmap = wx.StaticBitmap(clipboard_panel, -1, bmp2, (0, 0))

        file_bitmap.Bind(wx.EVT_LEFT_UP, self.on_clipboard)

        # cancel button
        cancel_button = wx.Button(self, -1, label='Cancel')
        cancel_button.Bind(wx.EVT_BUTTON, self.on_cancel)

        # layout
        grid = wx.GridBagSizer(10, 10)
        grid.Add(wx.StaticText(self, label="A device requested data from this Computer using Orange Share.\nYou can either select a file or send the current content of the clipboard.", size=(400, -1), pos=(0, 0), style=wx.ALIGN_CENTER), pos=(0, 0), span=(1, 2), flag=wx.CENTER)
        grid.Add(file_panel, pos=(1, 0), span=(1, 1), flag=wx.EXPAND)
        grid.Add(clipboard_panel, pos=(1, 1), span=(1, 1), flag=wx.EXPAND)
        grid.Add(cancel_button, pos=(2, 0), span=(1, 2), flag=wx.EXPAND)

        self.SetSizerAndFit(grid)
        self.Layout()
        self.Centre()
        self.Show()

    def on_file_click(self, e):
        config = Config.get_config()
        default_dir = config.config.get("GETDATA", "last_location", fallback="")

        self.dialog = wx.FileDialog(self, 'Send', style=wx.FD_OPEN, defaultDir=default_dir)
        if self.dialog.ShowModal() == wx.ID_OK:
            path = self.dialog.GetPath()
            self.result_data["filename"] = path
            logging.info("{} opened in get data dialog".format(path))
            config.config.set("GETDATA", "last_location", os.path.dirname(os.path.abspath(path)))
            config.save()
        else:
            path = None
            logging.info("no file opened in get data dialog".format(path))
        self.dialog.Destroy()
        self.Close(True)

    def on_clipboard(self, e):
        logging.info("got clipboard content")
        self.result_data["text"] = pyperclip.paste()
        self.Close(True)

    def on_cancel(self, e):
        self.Close(True)

    def try_destroying(self):
        if not self.dialog:
            try:
                self.Close(True)
                # updates the window for the close to take effect
                self.Disable()
            except:
                pass


class GetData(Resource):
    """
    Opens a short dialog and sends the selected data back to the device
    """

    def get(self):
        logging.info("data requested")

        result = {}

        app = wx.App()
        # app.locale = wx.Locale(wx.Locale.GetSystemLanguage())
        locale.setlocale(locale.LC_ALL,'C')
        frame = GetDataFrame(None, -1, "Send Data to Device", result_data=result)
        frame.Show()
        app.MainLoop()

        if "filename" in result:
            return send_file(result["filename"])
        elif "text" in result:
            response = make_response(result["text"], 200)
            response.mimetype = "text/plain"
            return response



import json
import logging
from flask_restful import Resource, reqparse

from orangeshare.devices import delete_device

parser = reqparse.RequestParser()
parser.add_argument('id',
                    type=str,
                    required=True,
                    help='provide the id')


class DeleteDevice(Resource):
    """
    Adds a new device, sets the uuid, returns the content for the QR Code.
    """

    def post(self):
        args = parser.parse_args()
        id = args["id"]

        logging.info("Deleting device: \"{}\"".format(id))

        return delete_device(id)



import datetime
import logging
import werkzeug
from flask_restful import Resource, reqparse

import orangeshare.shortcuts.handlers.open
import orangeshare.shortcuts.handlers.save
import orangeshare.shortcuts.handlers.clipboard

parser = reqparse.RequestParser()
parser.add_argument('mode',
                    type=str,
                    required=True,
                    help='provide a mode')
parser.add_argument('file',
                    type=werkzeug.datastructures.FileStorage,
                    location='files',
                    required=True,
                    help='provide a file')


class File(Resource):
    """
    Receives a file from a shortcut and calls the respective function for open, save and clipboard
    """

    def post(self):
        args = parser.parse_args()

        filename = args["file"].filename
        if filename == "":
            filename = datetime.datetime.now().isoformat()

        logging.info("Received File \"{}\" for {}".format(filename, args["mode"]))

        if args["mode"] == "open":
            return orangeshare.shortcuts.handlers.open.handle_file(args["file"], filename)

        elif args["mode"] == "save":
            return orangeshare.shortcuts.handlers.save.handle_file(args["file"], filename)

        elif args["mode"] == "clipboard":
            return orangeshare.shortcuts.handlers.clipboard.handle_file(args["file"], filename)

        return {"message": "unknown mode \"{}\"".format(args["mode"])}, 400



from flask import render_template, send_from_directory
from orangeshare import Config


def favicon():
    return send_from_directory("logo", "white.ico")


def index():
    return render_template("index.html")


def devices():
    return render_template("devices.html")


def shortcuts():
    return render_template("shortcuts.html")


def settings():
    return render_template("settings.html", conf_file=Config.get_config().file)



import datetime
import logging
import os
import subprocess
import sys
import webbrowser
from typing import Optional

from orangeshare import Config
from orangeshare.notify import notify
from orangeshare.temp_dir import temp_dir


def open_url(url: str):
    """
    Opens a given URL in the browser

    :param url: The url
    """

    if url == "":
        logging.error("can't open URL: URL is empty")
        return {"message": "URL is empty"}, 400

    logging.info("opening URL \"{}\"".format(url))

    config = Config.get_config()
    if config.config.getboolean("OPEN", "notification", fallback=True):
        notify("Opening URL \"{}\"".format(url))

    webbrowser.open(url)

    return {'success': True}


def open_file(filename: str):
    """
    Opens the given file in the default application

    :param filename: The filename
    """

    if sys.platform == "win32":
        os.startfile(filename)
    else:
        opener = "open" if sys.platform == "darwin" else "xdg-open"
        subprocess.call([opener, filename], stdout=None)


def open_text(text: str, filename: Optional[str] = None):
    if not filename:
        filename = "{}.txt".format(datetime.datetime.now().isoformat())

    config = Config.get_config()
    if config.config.getboolean("OPEN", "notification", fallback=True):
        notify("Opening Text")

    path = os.path.join(temp_dir.name, filename)
    textfile = open(path, "w")
    textfile.write(text)
    textfile.close()
    open_file(path)



import os
from typing import Optional

import validators

from orangeshare import Config
from orangeshare.notify import notify
from orangeshare.shortcuts.handlers.open_helper import open_url, open_text, open_file
from orangeshare.temp_dir import temp_dir


def handle_file(file, filename: str):
    """
    Copies the file to clipboard by saving it to a temporary directory and then copying it

    :param file: The file
    :param filename: The filename
    :return: response for the request
    """

    config = Config.get_config()
    if config.config.getboolean("OPEN", "notification", fallback=True):
        notify("Opening File \"{}\"".format(filename))

    path = os.path.join(temp_dir.name, filename)
    file.save(path)
    open_file(path)

    return {'success': True}


def handle_text(text: str, filename: str, file_extension: Optional[str] = None):
    """
    Copies the given text to the clipboard

    :param text: The text
    :param file_extension: The filename
    :param file_extension: The extension of the file
    :return:
    """

    config = Config.get_config()

    if validators.url(text):
        if config.config.getboolean("OPEN", "notification", fallback=True):
            notify("Opening URL \"{}\"".format(text))
        return open_url(text)
    else:
        # Firefox pages are weird...
        text_split = text.split("\n")
        if len(text_split) == 2 and validators.url(text_split[1]):
            if config.config.getboolean("OPEN", "notification", fallback=True):
                notify("Opening URL \"{}\"".format(text))
            return open_url(text_split[1])

    if config.config.getboolean("OPEN", "notification", fallback=True):
        notify("Opening File \"{}.{}\"".format(filename, file_extension))
    open_text(text, "{}.{}".format(filename, file_extension))

    return {'success': True}



import logging
import socket

from flask_restful import Resource, reqparse
from netifaces import interfaces, ifaddresses, AF_INET

from orangeshare import Config

parser = reqparse.RequestParser()
parser.add_argument('interface',
                    type=str,
                    required=False,
                    help='provide the interface')
parser.add_argument('name',
                    type=str,
                    required=False,
                    help='provide the name of the host')


class Host(Resource):
    """
    Information about the host
    """

    def get(self):
        """
        Get a JSON of all the available network interfaces and their IPs as well as the hostname

        :return: The interfaces and IPs
        """

        ip_list = {}
        for interface in interfaces():
            ifaddrs = ifaddresses(interface)
            if AF_INET in ifaddrs:
                for link in ifaddrs[AF_INET]:
                    ip_list[interface] = link['addr']

        config = Config.get_config()

        if len(ip_list) > 0:
            active_interface = config.config.get("HOST", "interface")
            if active_interface not in ip_list:
                active_interface = next(iter(ip_list))
        else:
            active_interface = None

        name = config.config.get("HOST", "name") or socket.gethostname()

        return {"interfaces": ip_list, "active_interface": active_interface, "name": name, "port": config.api_port}

    def post(self):
        """
        Set the interface and hostname
        """

        args = parser.parse_args()

        config = Config.get_config()

        if "interface" in args and args["interface"] is not None:
            interface = args["interface"]
            config.config.set("HOST", "interface", interface)
            logging.info("Setting interface to {}".format(interface))

        if "name" in args and args["name"] is not None:
            name = args["name"]
            config.config.set("HOST", "name", name)
            logging.info("Setting name to {}".format(name))

        config.save()

        return {"success": True}
